' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_Basic_Tv_LargeFont_001.spin                                                              *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     Screen buffer array - Approximately 12800 bytes / 320 longs ( 40 x 16 )                         *
' *     16 Foreground / background colours                                                              *
' *     Uses in-built Rom Font                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

OBJ

  tv_const                      : "AiChip_Basic_Tv_Constants_001"

  tv_LargeFont_Driver           : "AiChip_Basic_Tv_LargeFont_Driver_001"

CON

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Driver Attributes                                                                               |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  COLUMNS                       = tv_LargeFont_Driver#COLUMNS
  ROWS_PAL                      = tv_LargeFont_Driver#ROWS_PAL
  ROWS_NTSC                     = tv_LargeFont_Driver#ROWS_NTSC
  COLOURS                       = 16
  
  FONT_CHARS                    = 256

  ROWS_MAX                      = ROWS_NTSC #> ROWS_PAL
  
  SCREEN_SIZE_CHARS             = COLUMNS * ROWS_MAX

  BYTES_NEEDED_PER_SCREEN_CHAR  = 2
  SCREEN_SIZE_LONGS             = SCREEN_SIZE_CHARS * BYTES_NEEDED_PER_SCREEN_CHAR / 4

  BYTES_NEEDED_PER_COLOUR       = 8
  COLOURPALETTE_SIZE_LONGS      = COLOURS * BYTES_NEEDED_PER_COLOUR / 4

  TV_PARAMS_SIZE                = 14

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     TV Modes                                                                                        |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  TV_MODE_NTSC                  = tv_const#TV_MODE_NTSC
  TV_MODE_PAL                   = tv_const#TV_MODE_PAL

  TV_MODE_LARGE_FONT            = tv_const#TV_MODE_LARGE_FONT
  TV_MODE_SMALL_FONT            = tv_const#TV_MODE_SMALL_FONT

  TV_MODE_FONT_0                = tv_const#TV_MODE_FONT_0
  TV_MODE_FONT_1                = tv_const#TV_MODE_FONT_1
  TV_MODE_FONT_2                = tv_const#TV_MODE_FONT_2
  TV_MODE_FONT_3                = tv_const#TV_MODE_FONT_3

  TV_MODE_FAST_UPDATE           = tv_const#TV_MODE_FAST_UPDATE
  TV_MODE_FLICKER_FREE          = tv_const#TV_MODE_FLICKER_FREE

  TV_MODE_COLOR                 = tv_const#TV_MODE_COLOR
  TV_MODE_COLOUR                = tv_const#TV_MODE_COLOUR
  TV_MODE_MONOCHROME            = tv_const#TV_MODE_MONOCHROME
  
  TV_MODE_INTERLACED            = tv_const#TV_MODE_INTERLACED
  TV_MODE_NON_INTERLACED        = tv_const#TV_MODE_NON_INTERLACED

  TV_MODE_COMPOSITE             = tv_const#TV_MODE_COMPOSITE
  TV_MODE_BASEBAND              = tv_const#TV_MODE_BASEBAND
  TV_MODE_BROADCAST             = tv_const#TV_MODE_BROADCAST

  TV_MODE_FORCED                = tv_const#TV_MODE_FORCED
  
' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Characters for drawing boxes                                                                    |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  CHR_TOP_LEFT                  = 159
  CHR_TOP_MID                   = 144
  CHR_TOP_RIGHT                 = 158
  CHR_LFT_BAR                   = 145
  CHR_BOT_LEFT                  = 157
  CHR_BOT_RIGHT                 = 156

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Characters for drawing logic analyser traces                                                    |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  CHR_BIT_0_TO_0                = 129
  CHR_BIT_0_TO_1                = 254
  CHR_BIT_1_TO_0                = 255
  CHR_BIT_1_TO_1                = 134
  
VAR

  long colorPalette[ COLOURS * 2 ]

  word ptrToScreenBuffer
  
  byte rows
  word lastRow

  word mode

  long tv_status     '0/1/2 = off/invisible/visible              read-only   (14 longs)
  long tv_enable     '0/non-0 = off/on                           write-only
  long tv_pins       '%pppmmmm = pin group, pin group mode       write-only
  long tv_mode       '%tccip = tile,chroma,interlace,ntsc/pal    write-only
  long tv_screen     'pointer to screen (words)                  write-only
  long tv_colors     'pointer to colors (longs)                  write-only
  long tv_cols       'horizontal tiles                           write-only
  long tv_rows       'vertical tiles                             write-only
  long tv_hx         'horizontal tile expansion                  write-only
  long tv_vx         'vertical tile expansion                    write-only
  long tv_ho         'horizontal offset                          write-only
  long tv_vo         'vertical offset                            write-only
  long tv_broadcast  'broadcast frequency (Hz)                   write-only
  long tv_auralcog   'aural fm cog                               write-only

' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB Open( setBasePin, setMode, setPtrToScreenBuffer, ptrToPalette ) | ptrToParams

  if mode <> setMode

    Close
    
    mode := setMode | TV_MODE_FORCED

    ptrToScreenBuffer := setPtrToScreenBuffer
    
    SetPalette(ptrToPalette)
    if mode & TV_MODE_PAL
      ptrToParams := @PAL_PARAMS
    else
      ptrToParams := @NTSC_PARAMS
    longmove(@tv_status, ptrToParams, TV_PARAMS_SIZE)
    tv_pins := (setBasePin & $38) << 1 | (setBasePin & 4 == 4) & %0101
    tv_screen := ptrToScreenBuffer
    tv_colors := @colorPalette
    rows := tv_rows
    lastRow := (rows-1) * COLUMNS
    if mode & TV_MODE_MONOCHROME
      tv_mode ^= %1100
    if mode & TV_MODE_NON_INTERLACED
      tv_mode &= ! %0010
    if mode & TV_MODE_BROADCAST
      tv_pins |= %0001
    Cls
    tv_LargeFont_Driver.Open(@tv_status)

  return mode
  
PUB Close

  if mode
    tv_LargeFont_Driver.Close
    mode := 0

' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling Routines                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB Cls

  wordfill( ptrToScreenBuffer, $220, ROWS_MAX * COLUMNS)

PUB Scroll

  if mode & TV_MODE_FLICKER_FREE
    WaitForSynch
  wordmove( ptrToScreenBuffer, ptrToScreenBuffer+(COLUMNS<<1), lastRow)         ' Scroll lines
  wordfill( ptrToScreenBuffer+(lastRow<<1), $220, COLUMNS)                      ' Clear new line

PUB PokeChar( row, col, colour, c )

  colour //= COLOURS
  c &= $FF
  
  if mode & TV_MODE_FLICKER_FREE
    WaitForSynch

  c := (colour << 1 + c & 1) << 10 + $200 + c & $FE

  word[ ptrToScreenBuffer + ( row * ( COLUMNS<<1) ) + (col<<1) ] := c
  
PRI WaitForSynch

  repeat while tv_status == 1
  repeat while tv_status == 2

' *******************************************************************************************************
' *                                                                                                     *
' *     Attribute Handling for Video Drivers                                                            *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetColourCount

  result := COLOURS

PUB GetColumnCount

  result := COLUMNS
    
PUB GetRowCount

  result := rows

PUB ForceAttributes( setMode )

  setMode &= ! constant( TV_MODE_FONT_1 | TV_MODE_FONT_2 | TV_MODE_FONT_3 ) ' Fonts not supported

  result := setMode
    
' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette Handling for Video Drivers                                                       *
' *                                                                                                     *
' *******************************************************************************************************

PUB SetPalette( ptrToPalette ) | i, back

  back := byte[ptrToPalette]
  repeat i from 0 to COLOURS-2
    SetPaletteColour(i,byte[ptrToPalette+i+1],back)
  SetPaletteColour(COLOURS-1,byte[ptrToPalette+1],byte[ptrToPalette+COLOURS])

PUB SetPaletteColour( colorIdx, fore, back )

   colorPalette[colorIdx << 1]     := fore << 24 + back << 16 + fore << 8 + back
   colorPalette[colorIdx << 1 + 1] := fore << 24 + fore << 16 + back << 8 + back

DAT

' *******************************************************************************************************
' *                                                                                                     *
' *     Low-Level driver Parameters                                                                     *
' *                                                                                                     *
' *******************************************************************************************************

NTSC_PARAMS   long    0               'status
              long    1               'enable
              long    0               'pins
              long    %10010          'mode
              long    0               'screen
              long    0               'colors
              long    COLUMNS         'ht
              long    ROWS_NTSC       'vt
              long    4               'hx
              long    1               'vx
              long    0               'ho
              long    0               'vo
              long    0               'broadcast
              long    0               'auralcog

PAL_PARAMS    long    0               'status
              long    1               'enable
              long    0               'pins
              long    %10011          'mode
              long    0               'screen
              long    0               'colors
              long    COLUMNS         'ht
              long    ROWS_PAL        'vt
              long    5               'hx
              long    1               'vx
              long    0               'ho
              long    0               'vo
              long    66_750_00       'broadcast
              long    0               'auralcog

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip_LargeFont_002.spin                                                                *
' *                                                                                                     *
' *******************************************************************************************************

